@@ -6,12 +6,19 @@ from urlparse import urlparse
 
 import httplib2
 
+__version__ = '0.10.0'
 debugging = True
 
 try:
-    import simplejson as json
+    import jsonlib2 as json
 except:
-    import json
+    try:
+        import yajl as json
+    except:
+        try:
+            import simplejson as json
+        except:
+            import json
 
 JSON_HEADERS = {"content-type": "application/json",
                 "accept"      : "application/json"}
@@ -69,8 +76,9 @@ class Httplib2Client(HttpClient):
 
         if http_override is None:
             if '@' in self.uri:
-                user, password = self.uri.replace('http://','').split('@')[0].split(':')
-                self.uri = 'http://'+self.uri.split('@')[1]
+                protocol = 'https://' if 'https' in self.uri else 'http://'
+                user, password = self.uri.replace(protocol,'').split('@')[0].split(':')
+                self.uri = protocol+self.uri.split('@')[1]
                 if cache is None:
                     cache = '.cache'
                 self.http = httplib2.Http(cache)
@@ -307,7 +315,7 @@ class Views(object):
     def __getattr__(self, name):
         if debugging:
             response = self.db.http.head(self.path+name+'/')
-        if not debugging or response.status == 200:
+        if not debugging or response.status == 200 or response.status == 304:
             setattr(self, name, Design(self.db, '_design/'+name))
             return getattr(self, name)
         else:
@@ -327,7 +335,8 @@ def createdb(arg):
     else:
         db = Database(arg)
     response = db.http.put('')
-    assert response.status == 201
+    if response.status is not 201:
+        raise CouchDBException(response.body) 
     return json.loads(response.body)
 
 def deletedb(arg):
@@ -336,7 +345,8 @@ def deletedb(arg):
     else:
         db = Database(arg)
     response = db.http.delete('')
-    assert response.status == 200
+    if response.status != 200:
+        raise CouchDBException(response.body)
     return json.loads(response.body)
 
 
@@ -354,15 +364,48 @@ class Database(object):
             self.http = http
         self.views = Views(self)
 
-    def get(self, id_):
-        """Get a single document by id from the database."""
-        response = self.http.get(id_)
+    def get(self, id_, rev=None):
+        """Get a single document by id and (optionally) revision from the database."""
+        id_ = urllib.quote(id_)
+        if rev is None:
+            response = self.http.get(id_)
+        else:
+            response = self.http.get(id_+"?rev="+rev)
         if response.status == 200:
             obj = dict( (str(k),v) for k,v in json.loads(response.body).iteritems() )
             return Document(obj, db=self)
         else:
             raise CouchDBDocumentDoesNotExist("No document at id "+id_)
 
+    def get_revs(self, id_, fetch=False):
+        """Get all revisions of a single document from the database.
+        Returns a generator for the revision names or the actual documents,
+        depending on whether fetch=True is given (defaults to False).
+        """
+        response = self.http.get(id_ + "?revs_info=true")
+        for rev in json.loads(response.body)["_revs_info"]:
+            if rev["status"] != "available":
+                continue
+            if fetch:
+                yield self.get(id_, rev=rev["rev"])
+            else:
+                yield rev["rev"]
+
+    def all_ids(self):
+        """List all documents ids in the database."""
+        response = self.http.get("_all_docs")
+        obj = dict( (str(k),v) for k,v in json.loads(response.body).iteritems() )
+        ids = []
+        for row in obj["rows"]:
+            ids.append(str(row["id"]))
+        return tuple(ids)
+
+    def exists(self):
+        response = self.http.get('')
+        if response.status == 404:
+            return False
+        return True
+
     def create(self, doc, all_or_nothing=False):
         """Create a document. Accepts any object that can be converted in to a dict.
         If multiple documents are passed they are handed off to the bulk document handler.
@@ -479,6 +522,8 @@ class Database(object):
 
             for view in os.listdir(directory):
                 v = {}
+                if view.startswith('.'):
+                    continue
                 if os.path.isfile(os.path.join(directory, view, 'map.'+ext)):
                     v['map'] = open(os.path.join(directory, view, 'map.'+ext), 'r').read()
                 if os.path.isfile(os.path.join(directory, view, 'reduce.'+ext)):
